unit ENGINE;
{ simulacni SW pro projekt PES }
interface

uses Sun,display, crt;

procedure run;
function getDT(v,e:real):real;
procedure getWater(t,V:real);
procedure updateTa;
function getQ(v,dT:real):real;
procedure loadDef;
procedure loadWater;

type
  TWaterDemand= record
                  Qty,Temp:real;
                end;
var
  ACC1,ACC2: real; { akumulovana energie }
  TT1,TT2: real; { theshold temperature }
  V1,V2: real; { volume of accumulator }
  S1,S2:real;  { plocha nadrzi }
  dT1,dT2: real;   { +temp }
  T1,T2: real; { start temp }
  TEQ: real; { total energy [kWh] per sq meter }
  HRE: real; { energy per 1 hour }
  LD: word; { left days }
  colar:real; { collector area }
  CPDQM: real; { Collector Power Disipation per Sq meter/K }
  XCHGdT: real;  { Exchanger dT [C] }
  Ta: real; { ambient temp }
  PZ: real; { power looses }
  ACCLOSS1,ACCLOSS2: real;
  ACCTempCond: real; { ACC insulation material conductivity W/K/m^2 }
  iL1,iL2: real; { insulation mass - length }
  EXP:real; { external power }
  USER: real; { delivered energy }
  LOST1,LOST2:real;
  UACC1,UACC2:real;
  COLOST: real;
  WaterDemand:array[0..23] of TWaterDemand;


implementation

uses inifile;

procedure run;
var
  j: byte;
  ch: char;
begin
  day:=1;
  month:=6;
  hour:=1;
  clrscr;
  iprintSun;
  iprintStat;
  iprintAcc1;
  iprintAcc2;


  geoWidth:=50;
  sklon:=30;
  azimut:=0;
  H:=300;
  Z:=3;
  albedo:=0.2;
  cloudy:=50; { percent cloudy days }

  colar:=5; { m^2 }


  LD:=0; ACC1:=0; ACC2:=0;
  PZ:=0;
  ACCLOSS1:=0;ACCLOSS2:=0;
  USER:=0;
  LOST1:=0; LOST2:=0;
  COLOST:=0;
  UACC1:=0; UACC2:=0;
  { ====== USER ========= }

  TT1:= 75;
  TT2:= 40;
  T1:= 10; dT1:=0;
  T2:= 10; dT2:=0;
  V1:= 200;
  V2:= 600;

  TEQ:=0;
  CPDQM:= 1.1; { W/K/m^2 }
  XCHGdT:= 5;
  Ta:= 20;
  S1:= 2.7; iL1:=0.35; { [m] }
  S2:= 4.5;iL2:=0.40; { [m] }
  ACCTempCond:=0.025;

  loadDef;
  loadWater;

  {============}
  repeat
    updateTa;

    {=======ACCUMULATORS========}
    if (T1+dT1)<TT1 then begin
      ACC1:= ACC1 + HRE;
      PZ:= (T1+dT1+XCHGdT-Ta)*CPDQM;
    end
    else begin
      ACC2:= ACC2 + HRE;
      PZ:= (T2+dT2+XCHGdT-Ta)*CPDQM;
    end;



    {=======ACCUMULATORS========}


    { =========Accu loss====== }
    { 1st Tank }
    ACCLOSS1:= (S1/iL1)*ACCTempCond*(T1+dT1-Ta);
    ACC1:= ACC1 - (ACCLOSS1/1000);
    dT1:= getDT(V1,ACC1);
    if (dT1+T1)>99 then begin
      ACC1:=getQ(99-T1,V1);
      dT1:= getDT(V1,ACC1);
    end;

    LOST1:= LOST1 + (ACCLOSS1/1000);
    { 2nd Tank }
    ACCLOSS2:= ((S2/iL2)*ACCTempCond)*(T2+dT2-Ta);
    ACC2:= ACC2 - (ACCLOSS2/1000);
    dT2:= getDT(V2,ACC2);
    if (dT2+T2)>99 then begin
      ACC2:=getQ(99-T2,V2);
      dT2:= getDT(V2,ACC2);
    end;
    LOST2:= LOST2 + (ACCLOSS2/1000);

    {=======odberatel==========}

    { morning}
    if WaterDemand[round(hour)].Qty>0 then begin
      getWater(WaterDemand[round(hour)].Qty,WaterDemand[round(hour)].Temp);
    end;
    {=======odberatel==========}


    if (dT2>90) or (dT1>90) then begin
      writeln('Internal Error');
      exit;
    end;

    {========SUN===========}
    { aproximuj 1 hodinu po 0.1hodine }
    HRE:=0;

    for j:=1 to 10 do begin
      hour:= hour+0.1;
      sunSim;
      if I=0 then PZ:=0;
      HRE:= HRE + (((I-PZ)/10000)*colar);
      if HRE<0 then HRE:=0; { system odpoji kolektor pri Tk<Taccu }
      TEQ:= TEQ + ((I/10000)*colar);
    end;

    if hour>=24 then begin
      inc(day);
      inc(LD);
      hour:=0;
     {
      printSun;
      printStat;
      printAcc1;
      printAcc2;
      }
    end;
    if day>=31 then begin
      inc(month);
      day:=1;
    end;
    if month>12 then begin
      month:=1;
    end;

    { kolektory }
    COLOST:= COLOST + (PZ/1000)*colar;

    {========SUN===========}


    printSun;
    printStat;
    printAcc1;
    printAcc2;

    delay(20);
    if keypressed then begin
      ch:=readkey;
      repeat until keypressed;
      ch:=readkey;
    end;

    if ((LD mod 365)=0) and (LD<>0) and (hour=0) then begin
      writeln('******** NEW YEAR *********');
      repeat until keypressed;
      ch:=readkey;
      iprintSun;
      iprintStat;
      iprintAcc1;
      iprintAcc2;
      {EXP:=0;}
    end;
  until ch='q';
end;

function getDT(v,e:real):real;
begin
  getDT:= (e*3600000)/(4180*v); { litry! }
end;

function getQ(v,dT:real):real;
begin
  getQ:= (4180*v*dT)/3600000;
end;

procedure getWater(t,V:real);
var
  dQ:real;
  n,j:word;
begin
  n:=trunc(V);
  for j:=1 to n do begin
   if t<=(T2+dT2+XCHGdT) then begin
     dQ:= getQ(t-T2,1);
     ACC2:= ACC2 - dQ;
     UACC2:= UACC2 + dQ; { used energy }
     USER:= USER+dQ;
   end else begin
   if t<=(T1+dT1+XCHGdT) then begin
    { voda proleze 2. zasobik s vlaznou vodou }
    dQ:= getQ(dT2-XCHGdT,1);
    ACC2:= ACC2 - dQ;
    UACC2:= UACC2 + dQ;
    USER:= USER+dQ;
    { proleze 1. s teplou }
    dQ:= getQ(t-T1-(dT2-XCHGdT),1);
    ACC1:= ACC1 - dQ;
    UACC1:= UACC1 + dQ;
    USER:= USER+dQ;
    end else begin
      { voda proleze 2. zasobik s vlaznou vodou }
      dQ:= getQ(dT2-XCHGdT,1);
      ACC2:= ACC2 - dQ;
      UACC2:= UACC2 + dQ;
      USER:= USER+dQ;
      { proleze 1. s teplou }
      dQ:= getQ(dT1-dT2-XCHGdT,1);
      ACC1:= ACC1 - dQ;
      UACC1:= UACC1 + dQ;
      USER:= USER+dQ;

      dQ:= getQ(t-(T1+dT1-XCHGdT),1);
      EXP:=EXP + dQ;
      USER:= USER+dQ;
    end;
   end;
   dT1:= getDT(V1,ACC1);
   dT2:= getDT(V2,ACC2);
  end;
end;

procedure updateTa;
begin
  case month of { praha }
    1 :Ta:= 2.2;
    2 :Ta:= 3.4;
    3 :Ta:= 6.5;
    4 :Ta:= 12.1;
    5 :Ta:= 16.6;
    6 :Ta:= 20.6;
    7 :Ta:= 22.5;
    8 :Ta:= 22.6;
    9 :Ta:= 19.4;
    10:Ta:= 13.8;
    11:Ta:= 7.3;
    12:Ta:= 3.5;
  end;
end;

procedure loadDef;
var
   f:text;
   param:string;
begin
  assign(f,'system.inf');
  {$I-}
  reset(f);
  if IOResult<>0 then  begin
    writeln('file system.inf not found.');
    halt(1);
  end;
  {$I+}
  while not eof(f) do begin
    readln(f,param);
    cmd(param);
    case c of
      _day:     day:= xint[0];
      _month:   month:= xint[0];
      _hour:    hour:= xreal[0];
      _geoWidth:geoWidth:= xint[0];
      _sklon:   sklon:=xreal[0];
      _azimut:  azimut:=xreal[0];
      _nadmVyska:H:=xreal[0];
      _Z:        Z:=xint[0];
      _albedo:   albedo:=xreal[0];
      _SunConst: Io:=xreal[0];
      _cloudy:   clouds:=xint[0];
      _LD:       LD:= xint[0];
      _ACC1:     ACC1:=xreal[0];
      _ACC2:     ACC2:=xreal[0];
      _USER:     USER:=xreal[0];
      _LOST1:    LOST1:=xreal[0];
      _LOST2:    LOST2:=xreal[0];
      _COLOST:   COLOST:=xreal[0];
      _UACC1:    UACC1:=xreal[0];
      _UACC2:    UACC2:=xreal[0];
      _TT1:      TT1:=xreal[0];
      _TT2:      TT2:=xreal[0];
      _T1:        T1:=xreal[0];
      _dT1:      dT1:=xreal[0];
      _T2:       T2:=xreal[0];
      _dT2:      dT2:=xreal[0];
      _V1:       V1:=xreal[0];
      _V2:       V2:=xreal[0];
      _TEQ:      TEQ:=xreal[0];
      _CPDQM:    CPDQM:=xreal[0];
      _XCHGdT:   XCHGdT:=xreal[0];
      _S1:       S1:=xreal[0];
      _iL1:      iL1:=xreal[0];
      _S2:       S2:=xreal[0];
      _iL2:      iL2:=xreal[0];
      _ACCTempCond:ACCTempCond:=xreal[0];
      _colar:    colar:=xreal[0];
    end;
  end;
  close(f);
end;

procedure loadWater;
var
   f:text;
   param:string;
   x:real;
   err:integer;
   j:byte;
begin
  assign(f,'water.inf');
  {$I-}
  reset(f);
  if IOResult<>0 then  begin
    writeln('file water.inf not found.');
    halt(1);
  end;
  {$I+}
  for j:=0 to 23 do begin
    if eof(f) then exit;
    readln(f,param);
    cmdStr:=param;
    odmazMezirky(cmdStr);
    cutIt;
    val(cmdCont[0],x,err);
    if err<>0 then writeln('water.inf error @line:',j+1);
    waterDemand[j].Qty:=x;
    val(cmdCont[1],x,err);
    if err<>0 then writeln('water.inf error @line:',j+1);
    waterDemand[j].Temp:=x;
  end;
  close(f);
end;


end.