unit sun;

{
 simulator slunce
 vypocitava energeticky tok [W/m^2]
}

interface
uses math;

procedure SunSim;

var
  day,month:byte;
  hour:real;
  geoWidth:byte;
  sklon: real;
  azimut: real;
  H: real;      { nadporska vyska [m] }
  Z: real;      { stupen znecisteni atmosfery }
  Io: real;     { slunecni konstanta 1360W/m^2 }
  IPn:   real;   { intenzita na kolmou plochu W/m^2, prime zareni }
  IP:   real;   { intenzita na kolektor W/m^2, prime zareni }
  ID:   real;   { intenzita na kolektor W/m^2, difuzni zareni }
  I:    real;   { intenzita ID+IP }
  albedo: real;
  cloudy: byte; {% days}
  cloudyDays: word;

  oldDay:byte;
  clouds: byte;


implementation

procedure SunSim;
var
  GW,tau: real;
  gama: real;   { uhel dopadu paprsku slunce }
  dekl: real;    { deklinace }
  vyskaS,azimutS:real;
  alfa: real;   { sklon kolektoru }
  as:   real;   { azimut kol. }
  epsi: real;
  IPh,IDh:real;

begin
  { casovy uhel slunce [rad] }
  tau:= rad((hour-12)*15); { 1h = 15deg, 0 = 12h = poledne }

  { deklinace [rad] }
  dekl:= 23.45*sin(rad(((0.98*day) + (29.7*month))-109));
  dekl:= rad(dekl);

  { vyska slunce nad obzorem [rad] }
  GW:= rad(geoWidth);
  vyskaS:= arcSin((sin(dekl)*sin(GW)) + (cos(dekl)*cos(GW)*cos(tau)));

  { azimut slunce [rad] }
  azimutS:= arcSin((cos(dekl)*sin(tau))/cos(vyskaS));

  { uhel dopadu paprsku slunce [rad] }
  alfa:= rad(sklon);
  as:= rad(azimut);
  gama:= arcCos((sin(vyskaS)*cos(alfa)) + (cos(vyskaS)*sin(alfa)*cos(azimutS-as)));

  { zareni - soucinitel epsilon }

  epsi:=9.38076*(sin(vyskaS)+SQRT(0.003+SQR(sin(vyskaS))));
  epsi:= epsi/(2.0015*(1-H*0.0001));
  epsi:= epsi + 0.91018;

  { prime zareni na kolmou plochu k paprskum }
  IPn:= Io*exp(((-1)*Z)/epsi);
  { prime zareni na kolektor }
  IP:= IPn*cos(gama);
  {==========}
  { intenzita primeho zareni na vodorovnou plochu }
  IPh:= IPn*sin(vyskaS);
  { intenzita difuzniho zareni na vodorovnou plochu }
  IDh:= 0.33*(Io-IPn)*sin(vyskaS);
  { intenzita difuzniho zareni (approx) }
  ID:= 0.5*(1+cos(alfa))*IDh + 0.5*albedo*(1-cos(alfa))*(IPh+IDh);

  if oldDay<>day then begin clouds:= random(101);
    if clouds<=cloudy then
      inc(cloudyDays);
    {writeln(clouds);}
  end;

  if clouds>cloudy then
    I:= ID+IP
  else
    I:= ID;

  if vyskaS<0 then begin { noc }
    ID:=0; IP:=0; I:=0;
  end;
  oldDay:=day;
end;

begin
  randomize;
  day:=20;
  month:=1;
  hour:=12;

  geoWidth:=50;
  sklon:=35;
  azimut:=0;
  H:=300;
  Z:=3;
  albedo:=0.2;
  Io:= 1360;
  cloudyDays:=0;
  oldDay:=0;
end.